/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    : adslTraps.c
 * Author       : Subbi
 * Date         : 17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      : Amazon
 * Block        : ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *                 1) Initializing the adslTraps module
 *                 2) Handling adslTraps generation
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */

/* ===========================================================================
 * Revision History:
 *
 * $Log$
 * ===========================================================================
 */

/** adslTraps.c
 *  This file shows how to send a notification from inside the
 *  agent.  In this case we simply sleep for 30 seconds and send it,
 *  then we sleep for 30 more and send it again.
 *  We do this through the snmp_alarm mechanisms. Normally, you would
 *  probably want to do something to test whether or not to send an alarm,
 *  based on the type of mib module you were creating.
 *
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
 */
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>

/** This files contains Trap includes & prototypes **/
#include "adslTraps.h"

/*
 * ===========================================================================
 *                           LOCAL FUNCTIONS
 * ===========================================================================
 */
#ifndef ENABLE_DRIVER_CALL
    int stub_ioctl(int fd, int cmd, unsigned short *flags);
#endif

/** Our Trap initialization routine **/
void
init_adslTraps(void)
{
    DEBUGMSGTL(("adsl-Traps",
                "initializing (setting callback alarm)\n"));
    snmp_alarm_register(30,     /* seconds */
                        SA_REPEAT,      /* repeat (every 30 seconds). */
                        send_adsl_notification,      /* our callback */
                        NULL    /* no callback data needed */
        );
}

/** here we send an SNMP trap to all trapsinks **/
void
send_adsl_notification(unsigned int clientreg, void *clientarg)
{
    unsigned short flags = 0;
    unsigned short test_flags = 0;
    size_t notification_oid_len = 0;

    /* Assign our notification OID to the snmpTrapOID.0 object */
    oid objid_snmptrap[] = { 1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0 };
    size_t objid_snmptrap_len = OID_LENGTH(objid_snmptrap);

    /* define the OIDs for the notification we're going to send */

    /* ADSL-LINE-MIB::adslAtucPerfLofsThreshTrap */
    oid AtucPerfLofs_oid[] = { SNMP_ADSL_ATUC_TRAPS_PREFIX,0,1 };

    /* ADSL-LINE-MIB::adslAtucPerfLossThreshTrap */
    oid AtucPerfLoss_oid[] = { SNMP_ADSL_ATUC_TRAPS_PREFIX,0,2 };

    /* ADSL-LINE-MIB::adslAtucPerfESsThreshTrap */
    oid AtucPerfESs_oid[]  = { SNMP_ADSL_ATUC_TRAPS_PREFIX,0,4 };

    /* ADSL-LINE-MIB::adslAtucRateChangeTrap */
    oid AtucRateChange_oid[] = { SNMP_ADSL_ATUC_TRAPS_PREFIX,0,5 };

    /* ADSL-LINE-MIB::adslAturPerfLofsThreshTrap */
    oid AturPerfLofs_oid[] = { SNMP_ADSL_ATUR_TRAPS_PREFIX,0,1 };

    /* ADSL-LINE-MIB::adslAturPerfLossThreshTrap */
    oid AturPerfLoss_oid[] = { SNMP_ADSL_ATUR_TRAPS_PREFIX,0,2 };

    /* ADSL-LINE-MIB::adslAturPerfLprsThreshTrap */
    oid AturPerfLprs_oid[] = { SNMP_ADSL_ATUR_TRAPS_PREFIX,0,3 };

    /* ADSL-LINE-MIB::adslAturPerfESsThreshTrap */
    oid AturPerfESs_oid[] = { SNMP_ADSL_ATUR_TRAPS_PREFIX,0,4 };

    /* ADSL-LINE-MIB::adslAturRateChangeTrap */
    oid AturRateChange_oid[] = { SNMP_ADSL_ATUR_TRAPS_PREFIX,0,5 };

    /* Create the variables which might be needed to be sent in the trap */
    /* Here we don't send any notifictaion variables */
    netsnmp_variable_list *notification_vars = NULL;

    /*  Here we invoke the driver function which checks whether
     *  any Thresholds have exceeded the configured values and
     *  thereby set the appropriate Trap flags
     */

#ifdef ENABLE_DRIVER_CALL
    Fd = open(ADSL_DEV_PATH, O_RDONLY);
    if (Fd <=0) {
        snmp_log (LOG_ERR, "Unable to open the device... exiting!\n");
        return;
    }
    ioctl(Fd, ADSL_ATUR_TRAPS, &flags);
#else
    stub_ioctl(3, ADSL_ATUR_TRAPS, &flags);
#endif

     /* Check which flags have been Set */
     /* Generate Traps for those flags */
     /* Do I need to clear the flag_bit once I send out a Trap ?? */

      test_flags = 256;
      while (test_flags) {
      notification_vars = NULL;
      switch (IS_FLAG_SET(&flags, test_flags)) {
        case  ATUC_PERF_LOFS_THRESH_FLAG:
        {
                notification_oid_len = OID_LENGTH(AtucPerfLofs_oid);

                /* add in the trap definition object */
                    snmp_varlist_add_variable(&notification_vars,
                                    objid_snmptrap, objid_snmptrap_len,
                                    ASN_OBJECT_ID,
                                    (u_char *) AtucPerfLofs_oid,
                                     notification_oid_len * sizeof(oid));

                /* send the trap out to all registered receivers */
                    DEBUGMSGTL(("adsl-Traps",
                    	"sending AtucPerfLofsThresh trap\n"));
                    send_v2trap(notification_vars);

                /* clear this bit in flags */
                CLR_FLAG(&flags, ATUC_PERF_LOFS_THRESH_FLAG);
        	snmp_free_varbind(notification_vars);
                break;
        }

        case  ATUC_PERF_LOSS_THRESH_FLAG:
        {
                notification_oid_len = OID_LENGTH(AtucPerfLoss_oid);

                /* add in the trap definition object */
                    snmp_varlist_add_variable(&notification_vars,
                                    objid_snmptrap, objid_snmptrap_len,
                                    ASN_OBJECT_ID,
                                    (u_char *) AtucPerfLoss_oid,
                                     notification_oid_len * sizeof(oid));

                /* send the trap out to all registered receivers */
                    DEBUGMSGTL(("adsl-Traps",
                   	 "sending AtucPerfLossThresh trap\n"));
                    send_v2trap(notification_vars);

                /* clear this bit in flags */
                CLR_FLAG(&flags, ATUC_PERF_LOSS_THRESH_FLAG);
        	snmp_free_varbind(notification_vars);
                break;
        }

        case  ATUC_PERF_ESS_THRESH_FLAG:
        {
                    notification_oid_len = OID_LENGTH(AtucPerfESs_oid);

                /* add in the trap definition object */
                    snmp_varlist_add_variable(&notification_vars,
                                    objid_snmptrap, objid_snmptrap_len,
                                    ASN_OBJECT_ID,
                                    (u_char *) AtucPerfESs_oid,
                                     notification_oid_len * sizeof(oid));

                /* send the trap out to all registered receivers */
                    DEBUGMSGTL(("adsl-Traps",
                    	"sending AtucPerfESsThresh trap\n"));
                    send_v2trap(notification_vars);

                /* clear this bit in flags */
                CLR_FLAG(&flags, ATUC_PERF_ESS_THRESH_FLAG);
        	snmp_free_varbind(notification_vars);
                break;
        }

        case  ATUC_RATE_CHANGE_FLAG:
        {
                    notification_oid_len = OID_LENGTH(AtucRateChange_oid);

                /* add in the trap definition object */
                    snmp_varlist_add_variable(&notification_vars,
                                    objid_snmptrap, objid_snmptrap_len,
                                    ASN_OBJECT_ID,
                                    (u_char *) AtucRateChange_oid,
                                     notification_oid_len * sizeof(oid));

                /* send the trap out to all registered receivers */
                    DEBUGMSGTL(("adsl-Traps",
                    	"sending AtucRateChange trap\n"));
                    send_v2trap(notification_vars);

                /* clear this bit in flags */
                CLR_FLAG(&flags, ATUC_RATE_CHANGE_FLAG);
        	snmp_free_varbind(notification_vars);
                break;
        }

        case  ATUR_PERF_LOFS_THRESH_FLAG:
        {
                    notification_oid_len = OID_LENGTH(AturPerfLofs_oid);

                /* add in the trap definition object */
                    snmp_varlist_add_variable(&notification_vars,
                                    objid_snmptrap, objid_snmptrap_len,
                                    ASN_OBJECT_ID,
                                    (u_char *) AturPerfLofs_oid,
                                     notification_oid_len * sizeof(oid));

                /* send the trap out to all registered receivers */
                    DEBUGMSGTL(("adsl-Traps",
                    	"sending AturPerfLofsThresh trap\n"));
                    send_v2trap(notification_vars);

                /* clear this bit in flags */
                CLR_FLAG(&flags, ATUR_PERF_LOFS_THRESH_FLAG);
        	snmp_free_varbind(notification_vars);
                break;
        }

        case  ATUR_PERF_LOSS_THRESH_FLAG:
        {
                    notification_oid_len = OID_LENGTH(AturPerfLoss_oid);

                /* add in the trap definition object */
                    snmp_varlist_add_variable(&notification_vars,
                                    objid_snmptrap, objid_snmptrap_len,
                                    ASN_OBJECT_ID,
                                    (u_char *) AturPerfLoss_oid,
                                     notification_oid_len * sizeof(oid));

                /* send the trap out to all registered receivers */
                    DEBUGMSGTL(("adsl-Traps",
                    	"sending AturPerfLossThresh trap\n"));
                    send_v2trap(notification_vars);

                /* clear this bit in flags */
                CLR_FLAG(&flags, ATUR_PERF_LOSS_THRESH_FLAG);
        	snmp_free_varbind(notification_vars);
                break;
        }

        case  ATUR_PERF_LPRS_THRESH_FLAG:
        {
                    notification_oid_len = OID_LENGTH(AturPerfLprs_oid);

                /* add in the trap definition object */
                    snmp_varlist_add_variable(&notification_vars,
                                    objid_snmptrap, objid_snmptrap_len,
                                    ASN_OBJECT_ID,
                                    (u_char *) AturPerfLprs_oid,
                                    notification_oid_len * sizeof(oid));

                /* send the trap out to all registered receivers */
                    DEBUGMSGTL(("adsl-Traps",
                    	"sending AturPerfLprsThresh trap\n"));
                    send_v2trap(notification_vars);

                /* clear this bit in flags */
                CLR_FLAG(&flags, ATUR_PERF_LPRS_THRESH_FLAG);
        	snmp_free_varbind(notification_vars);
                break;
        }

        case  ATUR_PERF_ESS_THRESH_FLAG:
        {
                    notification_oid_len = OID_LENGTH(AturPerfESs_oid);

                /* add in the trap definition object */
                    snmp_varlist_add_variable(&notification_vars,
                                    objid_snmptrap, objid_snmptrap_len,
                                    ASN_OBJECT_ID,
                                    (u_char *) AturPerfESs_oid,
                                     notification_oid_len * sizeof(oid));

                /* send the trap out to all registered receivers */
                    DEBUGMSGTL(("adsl-Traps",
                    	"sending AturPerfESsThresh trap\n"));
                    send_v2trap(notification_vars);

                /* clear this bit in flags */
                CLR_FLAG(&flags, ATUR_PERF_ESS_THRESH_FLAG);
        	snmp_free_varbind(notification_vars);
                break;
        }

        case  ATUR_RATE_CHANGE_FLAG:
        {
                    notification_oid_len = OID_LENGTH(AturRateChange_oid);

                /* add in the trap definition object */
                    snmp_varlist_add_variable(&notification_vars,
                                    objid_snmptrap, objid_snmptrap_len,
                                    ASN_OBJECT_ID,
                                    (u_char *) AturRateChange_oid,
                                     notification_oid_len * sizeof(oid));

                /* send the trap out to all registered receivers */
                    DEBUGMSGTL(("adsl-Traps",
	                "sending AturRateChange trap\n"));
                    send_v2trap(notification_vars);

        	/*free the created notification variable list */
                CLR_FLAG(&flags, ATUR_RATE_CHANGE_FLAG);
        	snmp_free_varbind(notification_vars);
                break;
        }
        default:
                break;
     }
     test_flags = (test_flags >> 1);

#ifdef ENABLE_DRIVER_CALL
    close(Fd);
#endif
     }
}

#ifndef ENABLE_DRIVER_CALL
/* This function is part of the MEI driver which Bing Tao's team is supposed to
   provide us with. This function basically checks & compares the threshold
   values and on exceeding the configured threshold, sets the appropriate
   Threshold flag
*/

int stub_ioctl(int fd, int cmd, unsigned short *flags)
{

 switch (cmd) {
    case ADSL_ATUR_TRAPS:
            SET_FLAG(flags, ATUC_PERF_LOSS_THRESH_FLAG);
            SET_FLAG(flags, ATUC_RATE_CHANGE_FLAG);
            SET_FLAG(flags, ATUR_PERF_ESS_THRESH_FLAG);
            CLR_FLAG(flags, ATUR_RATE_CHANGE_FLAG);
        break;
    }
    return 0;
}
#endif

